#include "StdAfx.h"
#include ".\rcmotion.h"

int RCMotion::RCMInit(char ComPort[])
{
	DCB dcb;
	DWORD dwErr;
	COMMTIMEOUTS CommTimeout;

	if ( ComPort != NULL ) {
		strncpy(cRCMCom,ComPort, 4);
		cRCMCom[4]='\0';
		hRCMCom = CreateFile(cRCMCom, GENERIC_READ | GENERIC_WRITE,	0,
							 NULL, OPEN_EXISTING,	NULL, NULL);
		if (hRCMCom == INVALID_HANDLE_VALUE) { return ERR_RCM_OPENPORT; }

		// Setup serial port
		{
			memset(&dcb,0,sizeof(DCB));

			dcb.DCBlength=sizeof(DCB);
			dcb.BaudRate = 250000;
			dcb.fBinary  = TRUE;
			dcb.fParity  = FALSE;
			dcb.fOutxCtsFlow = FALSE;
			dcb.fOutxDsrFlow = FALSE;
			//dcb.fDtrControl = DTR_CONTROL_ENABLE;
			dcb.fDsrSensitivity = FALSE;
			//dcb.fTXContinueOnXoff = TRUE;
			//dcb.fOutX = TRUE;
			//dcb.fInX = TRUE;
			dcb.fErrorChar = FALSE;
			dcb.fNull = FALSE;
			//dcb.fRtsControl = RTS_CONTROL_ENABLE;
			//dcb.fAbortOnError = TRUE;
			//dcb.fDummy2 =;
			//dcb.wReserved =;
			//dcb.XonLim = 2048;
			//dcb.XoffLim = 512;
			dcb.ByteSize = 8;
			dcb.Parity = NOPARITY;
			dcb.StopBits = ONESTOPBIT;
			//dcb.XonChar = 17;
			//dcb.XoffChar = 19;
			//dcb.ErrorChar = ;
			//dcb.EofChar = EOF;
			//dcb.EvtChar = ;
			//dcb.wReserved1 =;

			dwErr=SetCommState(hRCMCom,&dcb);
		}

		
		{
			CommTimeout.ReadIntervalTimeout=0;
			CommTimeout.ReadTotalTimeoutMultiplier=1;
			CommTimeout.ReadTotalTimeoutConstant=1;
			CommTimeout.WriteTotalTimeoutMultiplier=0;
			CommTimeout.WriteTotalTimeoutConstant=0;

			SetCommTimeouts(hRCMCom,&CommTimeout);
		}

	

		return ERR_RCM_NO_ERROR;
	} else {
		return ERR_RCM_INVALID_PORT;
	}
}

PWDCmd RCMotion::SetCmd(byte paddress, byte pcode)
{
	PWDCmd cmd;

	cmd.address=paddress;
	cmd.code=pcode;
	cmd.axis=0;

	cmd.checksum = ~(paddress + pcode) + 1;
	cmd.size=4;

	return cmd;
}

// paddress = [0|1] left channel or right channel, pcode = code, uwData
PWDCmd RCMotion::SetCmd(byte paddress, byte pcode, WORD wData1)
{
	PWDCmd cmd;

	cmd=SetCmd(paddress,pcode);
	cmd.data[0]=HIBYTE(wData1);
	cmd.data[1]=LOBYTE(wData1);

	cmd.checksum = ~(~(cmd.checksum-1)+ cmd.data[0] + cmd.data[1]) + 1;
	cmd.size = 6;

	return cmd;
}

//void PWDCmd::SetCmd(byte address, byte code, byte data, byte data)
PWDCmd RCMotion::SetCmd(byte paddress, byte pcode, WORD wData1, WORD wData2)
{
	PWDCmd cmd;

	cmd=SetCmd(paddress, pcode, wData1);

	cmd.data[2]=HIBYTE(wData2);
	cmd.data[3]=LOBYTE(wData2);

	cmd.checksum = ~(~(cmd.checksum - 1) + cmd.data[2] + cmd.data[3]) + 1;
	cmd.size = 8;

	return cmd;
}

PWDReply RCMotion::SendCmd(PWDCmd *cmd)
{
	DWORD dwWritten=0,dwRead=0;
	PWDReply reply;
	DWORD dwErr;

	reply.dwSize=0;
	memset(reply.data,0,PWMREPLYSIZE);

	if ( !nosend ) {	
		if ( dwErr=WriteFile(hRCMCom, cmd, cmd->size, &dwWritten, NULL) ) {
			dwErr=ReadFile(hRCMCom, reply.data, 2, &(reply.dwSize), NULL);
		} 
	}

	if ( debug ) {
		printf("dwErr=%d, size= %d, cmd=%2x %2x %2x %2x %2x %2x %2x %2x %2x %2x\n",
			dwErr,cmd->size, cmd->address, cmd->checksum, cmd->axis, cmd->code,
			cmd->data[0], cmd->data[1], cmd->data[2], cmd->data[3],
			cmd->data[4], cmd->data[5]);
		if ( !nosend ) {
			reply.PrintData();
		}
	}

	return reply;
}

PWDReply RCMotion::SendCmd(byte paddress, byte pcode, WORD wData1, WORD wData2)
{
	PWDCmd cmd;

	cmd=SetCmd(paddress, pcode, wData1, wData2);
	return SendCmd(&cmd);
}

PWDReply RCMotion::SendCmd(byte paddress, byte pcode, WORD wData1)
{
	PWDCmd cmd;

	cmd=SetCmd(paddress, pcode, wData1);
	return SendCmd(&cmd);
}

PWDReply RCMotion::SendCmd(byte paddress, byte pcode)
{
	PWDCmd cmd;

	cmd=SetCmd(paddress, pcode);
	return SendCmd(&cmd);
}

PWDReply RCMotion::SendCmd_DW(byte paddress, byte pcode, DWORD dwData)
{
	return SendCmd(paddress,pcode,HIWORD(dwData),LOWORD(dwData));
}

void RCMotion::WheelSetPower(WORD wValue)
{
	SendCmd(1,SetMotorCommand,wValue);
	SendCmd(0,SetMotorCommand,wValue);
}

void RCMotion::WheelUpdate()
{
	SendCmd(1,Update);
	SendCmd(0,Update);
}

// Initalize two wheels
void RCMotion::WheelInit()
{
	SendCmd(1,Reset);
	SendCmd(1,SetLimitSwitchMode,0x0000);
	SendCmd(1,SetMotorCommand,0x0000);
	SendCmd(1,SetProfileMode,0x0001);
	SendCmd_DW(1,SetAcceleration,0x000000B0);
	SendCmd_DW(1,SetDeceleration,0x000000B0);
	SendCmd_DW(1,SetJerk,0x03290ABB);

	SendCmd(0,Reset);
	SendCmd(0,SetLimitSwitchMode,0x0000);
	SendCmd(0,SetMotorCommand,0x0000);
	SendCmd(0,SetProfileMode,0x0001);
	SendCmd_DW(0,SetAcceleration,0x000000B0);
	SendCmd_DW(0,SetDeceleration,0x000000B0);
	SendCmd_DW(0,SetJerk,0x03290ABB);

	WheelUpdate();
}

void RCMotion::WheelSetVelocity(DWORD dwSpeed)
{
	DWORD dwNegSpeed = ~dwSpeed + 1;

	WheelSetPower(0x4CC0);
	SendCmd_DW(1,SetVelocity,dwNegSpeed);
	SendCmd_DW(0,SetVelocity,dwSpeed);
}

// +ve for left, -ve for right
void RCMotion::WheelSetTurn(DWORD dwSpeed)
{
	WheelSetPower(0x4CC0);
	SendCmd_DW(1,SetVelocity,dwSpeed);
	SendCmd_DW(0,SetVelocity,dwSpeed);
}

RCMotion::RCMotion()
{
}

RCMotion::~RCMotion(void)
{
}

DWORD PWDReply::Status()
{
	return data[STATUS];
}

DWORD PWDReply::CheckChecksum()
{
	byte sum=0;
	DWORD i;

	for ( i=0 ; i<dwSize ; i++ )
	{
		sum+=data[i];
	}

	return sum;
}

void PWDReply::PrintData()
{
	DWORD i;

	printf("Reply Status=%2x, Checksum=%2x, Data=",	data[STATUS], data[CHECKSUM]);
	for ( i=DATA_START ; i<dwSize ; i++ ){
		printf("%2x ",data[i]);
	}
	printf("\n");

}